﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Runtime.InteropServices;

namespace LockDownSvc
{
    public class LockDown
    {
        public LockDown()
        {
        }

        /// <summary>
        /// Initialize persistent directory and rollover log file.
        /// </summary>
        public static void InitializeLockDown()
        {
            if (Log.LogFile == null)
            {
                Log.LogFile = Path.Combine(s_appDataPath, @"LockDownLog.csv");
                WinFunc.ForegroundUser = WinFunc.GetUser();

                // DataManager = new FileMgr(s_appDataPath, ".cfg");
                DataManager = new RegMgr("LockDown");

                Config.Initialize(DataManager, s_appDataPath);

                if (!Directory.Exists(s_appDataPath))
                    Directory.CreateDirectory(s_appDataPath);

                const long MB = 1 << 20;
                Log.RollLogFile(MB);

                try
                {
	                s_shdMemory = new SharedMemory(s_shdMemoryName, s_shdMemorySize);
	                s_shdMemory.WriteInt64((Int64)DateTime.Now.ToBinary(), s_dateOff);
                }
                catch (Exception ex)
                {
                    Log.LogEvent("Shared memory error - " + ex.Message);
                }
            }
        }

        #region ==== Shared Memory commanding.
        /// <summary>
        /// Shared memory layout.
        /// </summary>
        private   const int s_dateOff = 0;
        protected const int s_cmdBgOff = 32;
        protected const int s_cmdBgSeq = 32 + 4;
        protected const int s_cmdFgOff = 64;
        protected const int s_cmdFgSeq = 64 + 4;
        protected const int s_userLenOff = 80;
        protected const int s_userDataOff = 100;
        private   const int s_shdMemorySize = 4096;

        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        struct CfgItem
        {
            public DateTime date;
            public TimeSpan left;
            public TimeSpan span;
            public TimeSpan elapsed;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 30)]
            public string user;
        }

        int LoadCfgToShdMem(IntPtr ptr, int arrayIdx, Config cfg)
        {
            int off = Marshal.SizeOf(typeof(CfgItem)) * arrayIdx;
            ptr = new IntPtr(ptr.ToInt64() + off);
            GCHandle pinnedCfg = GCHandle.Alloc(ptr, GCHandleType.Pinned);
            CfgItem cfgItem = (CfgItem)Marshal.PtrToStructure(pinnedCfg.AddrOfPinnedObject(), typeof(CfgItem));
            pinnedCfg.Free();

            cfgItem.user = cfg.User;
            cfgItem.date = cfg.Date;
            cfgItem.left = cfg.Left;
            cfgItem.span = cfg.Span;
            cfgItem.elapsed = cfg.Elapsed;

            return off;
        }

        protected void LoadAllCfgsToShdMem()
        {
            string[] users = WinFunc.GetUsers();

            s_shdMemory.WriteInt32(users.Length, s_userLenOff);

            IntPtr ptr = s_shdMemory.GetPointer();
            ptr = new IntPtr(ptr.ToInt64() + s_userDataOff);
            for (int i = 0; i < users.Length; i++)
            {
                Config cfg = new Config();
                cfg.LoadFor(users[i], DataManager);
                LoadCfgToShdMem(ptr, i, cfg);
            }

            System.Diagnostics.Debug.WriteLine("done loading all cfg into shd memory");
        }

        private static string s_shdMemoryName = @"Global\LockDownShm";
        private static SharedMemory s_shdMemory;

        // Commands sent by Background to foreground.,
        public enum Bg2FgCommands { eNone = 0, eHello = 2, eLogOff = 4, eAlmostToast = 6, eTest = 8 };
        public enum Fg2BgCommands { eNone = 0, eHello = 1, eLogOff = 3, eEndSession = 5, eLoadCfg = 7, eTest = 9 };
     
        public static byte ReadCommand(int off, ref int lastSeq)
        {
            try
            {
                int seq = s_shdMemory.ReadInt32(off + 4);
                if (seq != lastSeq)
                {
                    lastSeq = seq;
                    return s_shdMemory.ReadByte(off);
                }
            }
            catch { }  // shared memory not ready.

            return 0;
        }

        public static void WriteCommand(byte cmd, int off)
        {
            try
            {
                s_shdMemory.WriteByte(cmd, off);
                int seq = s_shdMemory.ReadInt32(off + 4);
                s_shdMemory.WriteInt32(seq + 1, off + 4);
            }
            catch { }   // shared memory not ready.
        }
        #endregion

        private static string s_appDataPath =
         System.Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) + @"\LockDown";

        public static DataMgr DataManager
        {
            get { return m_dataMgr.Clone(); }
            set { m_dataMgr = value; }
        }
        private static DataMgr m_dataMgr;


    }
}
