﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Media;

namespace LockDownSvc
{
    /// <summary>
    /// Parental Control running as a window service to limit daily user time.
    /// 
    /// Author: Dennis Lang with some support routines by Pascal Ganaye
    /// LockDown service - Dec 2010
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    public class LockDownFg : LockDown
    {
        private StatusDlg m_statusDlg = null;
        private System.Timers.Timer m_sysTimer = new System.Timers.Timer(1000);
        private int m_seq = -1;

        #region ==== Public interface
        public LockDownFg()
        {
            Action = Actions.eNone;

            InitializeLockDown();
            WinFunc.ForegroundUser = WinFunc.GetUser();

            Configuration = new Config();
            LoadConfig();
            Action = Actions.eNone;

            // Timer to check if time to force user to logoff.
            m_sysTimer.Elapsed += new System.Timers.ElapsedEventHandler(Timer_tick);
            m_sysTimer.Enabled = true;
         
            StatusDialog = new StatusDlg(this);
            string msg = string.Format("Watching '{0}'", WinFunc.ForegroundUser);
            SetStatusMsg(msg, 0.5f);

            Application.Run(StatusDialog);
        }

        public StatusDlg StatusDialog
        {
            get { return m_statusDlg; }
            set { m_statusDlg = value; }
        }

        /// <summary>
        /// Asyncrhonouse actions - allows threads to communication to request an action.
        /// </summary>
        public enum Actions { eNone, eEndSession, ePause, eContinue };
        public Actions Action { get; set; }

        #endregion

        #region ===== Timer and Action commands
        /// <summary>
        /// Timer to execute asynchronous actions and check if time to log user off.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal void Timer_tick(object sender, System.Timers.ElapsedEventArgs e)
        {
            switch (ReadFgCommand(ref m_seq))
            {
                case Bg2FgCommands.eHello:
                    WriteBgCommand(Fg2BgCommands.eHello);
                    break;

                case Bg2FgCommands.eAlmostToast:
                    LoadConfig();
                    SetStatusMsg("Logging Off", 1.0f);
                    m_statusDlg.Flash = true;
                    break;

                case Bg2FgCommands.eLogOff:
                    WinExitWindows.ShutDown(
                        WinExitWindows.ShutDownMethod.LogOff | WinExitWindows.ShutDownMethod.ForceIfHung,
                        WinExitWindows.ShutDownReason.MajorOperatingSystem | WinExitWindows.ShutDownReason.FlagPlanned);
                    break;
            }

            switch (Action)
            {
                case Actions.eEndSession:
                    WriteBgCommand(Fg2BgCommands.eEndSession);
                    break;

                case Actions.ePause:
                    return;

                case Actions.eContinue:
                    Configuration.Date = DateTime.Now;
                    Action = Actions.eNone;
                    break;
            }
        }

        public static Bg2FgCommands ReadFgCommand(ref int lastSeq)
        {
            return (Bg2FgCommands)ReadCommand(s_cmdFgOff, ref lastSeq);
        }
        public static void WriteBgCommand(Fg2BgCommands cmd)
        {
            WriteCommand((byte)cmd, s_cmdBgOff);
        }


        SoundPlayer m_player = new SoundPlayer();
        private void PlaySound()
        {
            m_player.Stream = Properties.Resources.sonar;
            m_player.Play();
        }

        #endregion

        #region ==== User Configuration
        public Config Configuration  { get; set; }

        /// <summary>
        /// Load configuration.
        /// </summary>
        private void LoadConfig()
        {
            if (!Configuration.LoadFor(WinFunc.ForegroundUser, DataManager))
            {
                Configuration = new Config();
            }
        }

        #endregion

        #region ===== Dialog messages

        /// <summary>
        /// Set message in status dialog. 
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="opacity"></param>
        private void SetStatusMsg(string msg, float opacity)
        {
            m_statusDlg.SetMsg(msg, opacity);
            Log.LogMessage("Status, " + msg + Configuration.ToString());
            PlaySound();
        }

        #endregion

    }
}
