using System;
using System.Collections.Generic;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace LockDownSvc
{
    static class Program
    {
        [DllImport("kernel32.dll")]
        static extern bool AttachConsole(int dwProcessId);
        private const int ATTACH_PARENT_PROCESS = -1;

        /// <summary>
        /// Parental Control running as a window service to limit daily user time.
        /// 
        /// Author: Dennis Lang with some support routines by Pascal Ganaye
        /// LockDown service - Dec 2010
        /// http://home.comcast.net/~lang.dennis/
        /// </summary>
        [STAThread]
        static void Main()
        {
            bool startSvc = true;
            string[] args = Environment.GetCommandLineArgs();
            if (args.Length > 1)
            {
                // Redirect console output to parent process.
                // Must be before any calls to Console.WriteLine().
                AttachConsole(ATTACH_PARENT_PROCESS);
                LockDown.InitializeLockDown();
            }

            // Handle optional command line switches:
            //
            //  -about  
            //  -gui            ; used by service to run foreground Graphic User Interface.  
            //  -install
            //  -uninstall
            // [Debug]
            //  -unlimit        ; set current user configuration to unlimited time span and time left.
            //  -minutes <mm>   ; set current user configuration to have mm minutes left.
           
            //  
            for (int i = 1; i < args.Length; i++)
            {
                Log.LogEvent(args[i]);
                string arg = args[i].ToLower();
                
                if (arg ==  "-about")
                {
                    Console.WriteLine(Application.ProductName + " " + Application.ProductVersion);
                    Console.WriteLine("LockDown - Dennis Lang - Dec 2010");
                    Console.WriteLine("http://home.comcast.net/~lang.dennis/");
                }
                else if (arg == "-gui")
                {
                    // Thread.CurrentThread.TrySetApartmentState(ApartmentState.STA);
                    // Run as an application rather than a service.
                    LockDownFg lockDownFg = new LockDownFg();
                    startSvc = false;
                }
#if false
                else if (arg == "-unlimited")
                {
                    // Reset configuration to unlimited time.
                    Config m_cfg = new Config();
                    m_cfg.SetToUnlimited();
                    try
                    {
                        m_cfg.SaveFor(WinFunc.ForegroundUser, LockDown.DataManager);
                        Log.LogEvent("unlimited");
                    }
                    catch (Exception ex)
                    {
                        Log.LogEvent("unlimited save error - " + ex.Message);
                    }
                    startSvc = false;
                }
                else if ((arg == "-minutes")  && i+1 < args.Length)
                {
                    // Set configuration to 2 minutes for testing.
                    i++;

                    Config m_cfg = new Config();
                    m_cfg.Left = m_cfg.Span = new TimeSpan(0, int.Parse(args[i]), 0);
                    m_cfg.Date = DateTime.Now;

                    try
                    {
                        m_cfg.SaveFor(WinFunc.ForegroundUser, LockDown.DataManager);
                        Log.LogEvent("args set span to " + m_cfg.SpanString());
                    }
                    catch (Exception ex)
                    {
                        Log.LogEvent(ex.Message);
                    }
                    startSvc = false;
                }
#endif
                else if (arg == "-install")
                {
                    IntSvcInstaller.InstallService("LockDownSvc", "LockDown", "Limit User Time", true);
                    return;
                }
                else if (arg == "-uninstall")
                {
                    IntSvcInstaller.UnInstallService("LockDownSvc", true);
                    return;
                }
            }

            if (startSvc)
            {
                ServiceBase[] ServicesToRun;
                ServicesToRun = new ServiceBase[] { new LockDownSvc() };
                ServiceBase.Run(ServicesToRun);
            }
        }
    }
}