﻿
/*
 * ServiceInstaller
 * Copyright (C)2007 Victor Baidoon <v_baidoon@yahoo.com>
 * Last modified: Jan 07, 2008
 * All Rights Reserved
 * 
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

namespace LockDownSvc
{
    using System;
    using System.Collections.Generic;
    using System.Runtime.InteropServices;
    using System.Text;
    using System.ServiceProcess;
    using System.Management;
    using System.Windows.Forms;

    /// <summary>
    /// Summary description for ServiceInstaller.
    /// </summary>
    public class IntSvcInstaller
    {
        #region Import from kernel32.dll
        [DllImport("kernel32.dll")]
        private static extern bool AttachConsole(int dwProcessId);
        private const int ATTACH_PARENT_PROCESS = -1;
        #endregion Import from kernel32.dll

        #region Import from advapi32.dll
        [DllImport("advapi32.dll")]
        public static extern IntPtr OpenSCManager(string lpMachineName, string lpSCDB, uint scParameter);
        [DllImport("advapi32.dll")]
        public static extern IntPtr CreateService(IntPtr SC_HANDLE, string lpSvcName, string lpDisplayName,
          uint dwDesiredAccess, uint dwServiceType, uint dwStartType, uint dwErrorControl, string lpPathName,
          string lpLoadOrderGroup, IntPtr lpdwTagId, string lpDependencies, string lpServiceStartName, string lpPassword);

        public enum SERVICE_INFO_LEVEL : uint
        {
            SERVICE_CONFIG_DESCRIPTION = 1,
            SERVICE_CONFIG_FAILURE_ACTIONS = 2,
        }

        public enum SC_ACTION_TYPE
        {
            None = 0,
            RestartService = 1,
            RebootComputer = 2,
            Run_Command = 3
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct SC_ACTION
        {
            public SC_ACTION_TYPE Type;
            public uint Delay;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct SERVICE_FAILURE_ACTIONS
        {
            public int dwResetPeriod;
            [MarshalAs(UnmanagedType.LPTStr)]
            public string lpRebootMsg;
            [MarshalAs(UnmanagedType.LPTStr)]
            public string lpCommand;
            public int cActions;
            public IntPtr lpsaActions;
        }

        [DllImport("advapi32.dll")]
        public static extern bool ChangeServiceConfig2A(
            IntPtr SVHANDLE, SERVICE_INFO_LEVEL dwInfoLevel, ref string lpInfo);
        [DllImport("advapi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool ChangeServiceConfig2(
            IntPtr hService,
            int dwInfoLevel,
            IntPtr lpInfo);

        [DllImport("advapi32.dll")]
        public static extern void CloseServiceHandle(IntPtr SC_HANDLE);
        [DllImport("advapi32.dll", SetLastError = true)]
        public static extern IntPtr OpenService(IntPtr SC_HANDLE, string lpSvcName, uint dwDesiredAccess);
        [DllImport("advapi32.dll")]
        public static extern int DeleteService(IntPtr SC_HANDLE);
        #endregion Import from advapi32.dll

        public static bool InstallService(
            // string svcPath, 
            string svcName, 
            string svcDispName, 
            string svcDescription, 
            bool bRun)
        {
            string svcPath = Application.ExecutablePath;

            try
            {
                UnInstallService(svcName, false);

                // Open Service Control Manager
                IntPtr hSCM = OpenSCManager(
                    null, // lpMachineName
                    null, // lpSCDB
                    (uint)WinFunc.SCM_ACCESS_TYPE.SC_MANAGER_CREATE_SERVICE // scParameter
                    );

                if (hSCM == IntPtr.Zero)
                {
                    Console.WriteLine("OpenSCManager failed, error " + Marshal.GetLastWin32Error());
                    return false;
                }

                // Create the service
                IntPtr hService = CreateService(
                    hSCM,       // SC_HANDLE
                    svcName,    // lpSvcName
                    svcDispName,// lpDisplayName
                    WinFunc.SERVICE_ALL_ACCESS,                           // dwDesiredAccess
                    (uint)WinFunc.SERVICE_TYPE.SERVICE_WIN32_OWN_PROCESS, // dwServiceType
                    (uint)WinFunc.SERVICE_START_TYPE.SERVICE_AUTO_START,  // dwStartType
                    (uint)WinFunc.SERVICE_ERROR_CODE.SERVICE_ERROR_NORMAL,// dwErrorControl
                    svcPath,    // lpPathName
                    null,       // lpLoadOrderGroup
                    IntPtr.Zero,// lpdwTagId
                    "SENS\0COMSysApp\0",       // lpDependencies      
                    null,       // lpServiceStartName
                    null        // lpPassword
                    );
                if (hService == IntPtr.Zero)
                {
                    Console.WriteLine("CreateService failed, error " + Marshal.GetLastWin32Error());
                    CloseServiceHandle(hSCM);
                    return false;
                }

                SC_ACTION action = new SC_ACTION();
                action.Type = SC_ACTION_TYPE.RestartService;    
                action.Delay = (uint)TimeSpan.FromMinutes(1).TotalMilliseconds;

                IntPtr lpsaActions = Marshal.AllocHGlobal(Marshal.SizeOf(action) * 2);
                if (lpsaActions == IntPtr.Zero)
                {
                    Console.WriteLine("AllocHGlobal failed, error " + Marshal.GetLastWin32Error());
                    CloseServiceHandle(hSCM);
                    return false;
                }

                Marshal.StructureToPtr(action, lpsaActions, false);

                IntPtr nextAction = (IntPtr)(lpsaActions.ToInt64() + Marshal.SizeOf(action));
                action.Type = SC_ACTION_TYPE.None;
                Marshal.StructureToPtr(action, nextAction, false);

                SERVICE_FAILURE_ACTIONS failureActions = new SERVICE_FAILURE_ACTIONS();
                failureActions.dwResetPeriod = (int)TimeSpan.FromMinutes(5).TotalSeconds;
                failureActions.lpRebootMsg = "";
                failureActions.lpCommand = "";
                failureActions.cActions = 2;
                failureActions.lpsaActions = lpsaActions;

                IntPtr lpInfo = Marshal.AllocHGlobal(Marshal.SizeOf(failureActions));
                if (lpInfo == IntPtr.Zero)
                {
                    Console.WriteLine("AllocHGlobal failed, error " + Marshal.GetLastWin32Error());
                    Marshal.FreeHGlobal(lpsaActions);
                    CloseServiceHandle(hSCM);
                    return false;
                }

                Marshal.StructureToPtr(failureActions, lpInfo, false);

                bool okay = ChangeServiceConfig2(hService, (int)SERVICE_INFO_LEVEL.SERVICE_CONFIG_FAILURE_ACTIONS, lpInfo);
                Marshal.FreeHGlobal(lpInfo);
                Marshal.FreeHGlobal(lpsaActions);

#if true
                // Change service description
                okay = okay && ChangeServiceConfig2A(
                    hService,                   // handle to service
                    SERVICE_INFO_LEVEL.SERVICE_CONFIG_DESCRIPTION, // change: description
                    ref svcDescription);                    // value: new description
#endif
                CloseServiceHandle(hSCM);

                // Set check box "Allow service to interact with desktop" to checked.
                OperatingSystem os = Environment.OSVersion;
                // if (os.Version.Major < 6) // do not do it on Vista
                                            // no error on Win7
                {
                    System.ServiceProcess.ServiceController serviceController = new System.ServiceProcess.ServiceController();
                    serviceController.ServiceName = svcName;

                    ConnectionOptions coOptions = new ConnectionOptions();
                    coOptions.Impersonation = ImpersonationLevel.Impersonate;

                    ManagementScope mgmtScope = new System.Management.ManagementScope(@"root\CIMV2", coOptions);
                    mgmtScope.Connect();

                    ManagementObject wmiService = new ManagementObject("Win32_Service.Name='" + svcName + "'");

                    ManagementBaseObject InParam = wmiService.GetMethodParameters("Change");
                    InParam["DesktopInteract"] = true;

                    ManagementBaseObject OutParam = wmiService.InvokeMethod("Change", InParam, null);

                    serviceController.Start();
                }

                Console.WriteLine("Install completed");

                // Run service
                if (bRun) 
                    return RunService(svcName);
                
                return true;
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        public static bool UnInstallService(string svcName, bool reportErrors)
        {
            // Stop the service if it's running first
            try
            {
                KillService(svcName);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                if (reportErrors)
                	Console.WriteLine("Kill service failed " + ex.Message);
            }

            // Open Service Control Manager
            IntPtr hSCM = OpenSCManager(
                null,
                null,
                (uint)WinFunc.ACCESS_TYPE.GENERIC_WRITE);
            if (hSCM == IntPtr.Zero)
            {
                if (reportErrors)
                	Console.WriteLine("OpenSCManager failed, error " + Marshal.GetLastWin32Error());
                return false;
            }

            // Open the service
            IntPtr hService = OpenService(
                hSCM,
                svcName,
                (uint)WinFunc.ACCESS_TYPE.DELETE);
            if (hService != IntPtr.Zero)
            {
                // Uninstall the service
                int i = DeleteService(hService);
                if (i != 0)
                {
                    CloseServiceHandle(hSCM);
                    if (reportErrors)
                    	Console.WriteLine("Service uninstalled");
                    return true;
                }
                else
                {
                    if (reportErrors)
                    	Console.WriteLine("DeleteService failed, error " + Marshal.GetLastWin32Error());
                    CloseServiceHandle(hSCM);
                    return false;
                }
            }
            else
            {
                CloseServiceHandle(hSCM);
                if (reportErrors)
                	Console.WriteLine("Service not present, nothing uninstalled");
                return false;
            }
        }

        internal static bool RunService(string svcName)
        {
            ServiceController controller = new ServiceController();
            controller.MachineName = ".";
            controller.ServiceName = svcName;
            if (controller.Status == ServiceControllerStatus.Stopped)
            {
                // Start the service
                controller.Start();
            }
            return true;
        }

        internal static bool KillService(string svcName)
        {
            ServiceController controller = new ServiceController();
            controller.MachineName = ".";
            controller.ServiceName = svcName;
            if (controller.Status == ServiceControllerStatus.Running)
            {
                // Stop the service
                controller.Stop();
            }
            return true;
        }

        internal static bool IsServiceInstalled(string svcName)
        {
            // Open Service Control Manager
            IntPtr hSCM = OpenSCManager(
                null,
                null,
                (uint)WinFunc.SERVICE_ACCESS_TYPE.SERVICE_START);
            if (hSCM == IntPtr.Zero)
                return false;

            // Open the service
            IntPtr hService = OpenService(
                hSCM,
                svcName,
                (uint)WinFunc.ACCESS_TYPE.STANDARD_RIGHTS_EXECUTE);
            CloseServiceHandle(hSCM);
            if (hService != IntPtr.Zero)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private static void ShowHelp()
        {
            AttachConsole(ATTACH_PARENT_PROCESS); // use this if OutputType is WindowsApplication
            // Console.WriteLine("...");
        }
    }
}
