﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;

namespace LockDownSvc
{
    /// <summary>
    /// Log messages (events) to windows event logging subsystem.
    /// 
    /// Author: Dennis Lang
    /// LockDown service - Dec 2010
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    class Log
    {
        #region ==== Log Events to Windows Event service.
        private static EventLog m_eventLog;
        private static string EventLogSource = "LockDown";
        private static string EventLogName = string.Empty; // "LockDown";

        public static void LogEvent(string msg)
        {
            try
            {
                if (m_eventLog == null)
                {
                    if (!System.Diagnostics.EventLog.SourceExists(EventLogSource))
                        System.Diagnostics.EventLog.CreateEventSource(EventLogSource, EventLogName);

                    m_eventLog = new EventLog(EventLogName);
                    m_eventLog.Source = EventLogSource;
                    m_eventLog.MaximumKilobytes = 128;       // 64 smallest value allowed.
                    m_eventLog.ModifyOverflowPolicy(OverflowAction.OverwriteOlder, 2);
                }

                string user = WinFunc.ForegroundUser;
                try
                {
                    m_eventLog.WriteEntry("[" + user + "] " + msg);
                }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }  
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }

        #endregion

        #region ===== Log Messages to file.
        public static string LogFile { get; set; }

        public static void LogMessage(string msg)
        {
            string user = WinFunc.ForegroundUser;
            string logMsg = string.Format("{0} {1},[{2}],{3}",
                DateTime.Now.ToShortDateString(),
                DateTime.Now.ToString("HH:MM:ss"), 
                user, msg);

            try
            {
                bool writeHeader = !File.Exists(LogFile);
                StreamWriter logStream = new StreamWriter(
                    File.Open(LogFile, FileMode.Append, FileAccess.Write, FileShare.ReadWrite));
                // StreamWriter logStream = File.AppendText(LogFile);
                if (writeHeader)
                    logStream.WriteLine("Date, User, Action, Message");
                logStream.WriteLine(logMsg);
                logStream.Close();
                System.Diagnostics.Debug.WriteLine(logMsg);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// If Log file is too large, roll over to previous log file.
        /// </summary>
        public static void RollLogFile(long maxSize)
        {
            try
            {
                // Check size of log file, rotate if too large.
                FileInfo fileInfo = new FileInfo(LogFile);

                if (fileInfo.Exists && fileInfo.Length > maxSize)
                {
                    string rollToFile = "Prev" + LogFile;
                    if (System.IO.File.Exists(rollToFile))
                        System.IO.File.Delete(rollToFile);
                    System.IO.File.Move(LogFile, rollToFile);
                }
            }
            catch (Exception ex)
            {
                Log.LogEvent("RollLog error - " + ex.Message);
            }
        }
        #endregion
    }
}
