using System;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.ServiceProcess;
using System.Configuration;
using System.Configuration.Install;

using Microsoft.Win32;


namespace LockDownSvc
{
    /// <summary>
    /// Logic to install program as a windows Service.
    /// Use windows framework InstallUtil to install as a windows service.
    /// 
    /// Example:
    /// %WinDir%\Microsoft.NET\Framework\v2.0.50727\InstallUtil.exe LockDownSvc.exe
    /// 
    /// Used by: Dennis Lang
    /// LockDown service - Dec 2010
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    [RunInstaller(true)]
    public class SvcInstaller : Installer
    {
        private static readonly string SVC_NAME = "LockDownSvc";    // ** Must match Srv class name **
        private static readonly string SVC_DESC = "LockDown - Limit User Time";

        public SvcInstaller()
        {
            try
            {
                Installers.Clear();

                ServiceInstaller serviceInstaller = new ServiceInstaller();
                serviceInstaller.StartType = ServiceStartMode.Automatic;
                serviceInstaller.ServiceName = SVC_NAME;
                serviceInstaller.DisplayName = "LockDown";
                serviceInstaller.Description = SVC_DESC;
                serviceInstaller.ServicesDependedOn = new string[] { "SENS", "COMSysApp" };

                Installers.Add(serviceInstaller);

                ServiceProcessInstaller processInstaller = new ServiceProcessInstaller();
                processInstaller.Account = ServiceAccount.LocalSystem;
                processInstaller.Password = null;
                processInstaller.Username = null;

                Installers.Add(processInstaller);

                LockDown.InitializeLockDown();
                DataMgr dataMgr = LockDown.DataManager;
                dataMgr.SetChild(Config.InternalKeyStr);
                dataMgr.SetString(Config.AdminUserKeyStr, WinFunc.ForegroundUser);
                dataMgr.Apply();
            }
            catch (Exception ex)
            {
                Log.LogEvent("Svc Setup error - " + ex.Message);
            }
        }

        protected override void OnAfterInstall(IDictionary savedState)
        {
            ServiceController controller = null;
            ServiceController[] controllers = ServiceController.GetServices();
            for (int i = 0; i < controllers.Length; i++)
            {
                if (controllers[i].ServiceName == SVC_NAME)
                {
                    controller = controllers[i];
                    break;
                }
            }

            if (controller == null)
            {
                Log.LogEvent("Svr Install failed");
                return;
            }

            // if the service is not active, start it
            if (controller.Status != ServiceControllerStatus.Running)
            {
                string[] args = { "-install" };
                try
                {
                    controller.Start(args);
                }
                catch (Exception ex)
                {
                    Log.LogEvent("Svr Install failed - " + ex.Message);
                }
            }
        }
    }
}
