﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace LockDownSvc 
{
    /// <summary>
    /// Measure idle time (seconds between input events).
    /// 
    /// 
    /// Author: Dennis Lang
    /// LockDown service - Dec 2010
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    class WinIdle
    {
        [StructLayout(LayoutKind.Sequential)]
        struct LASTINPUTINFO
        {
            public static readonly int SizeOf = Marshal.SizeOf(typeof(LASTINPUTINFO));

            [MarshalAs(UnmanagedType.U4)]
            public int cbSize;
            [MarshalAs(UnmanagedType.U4)]
            public UInt32 dwTime;
        }

        [DllImport("user32.dll")]
        static extern bool GetLastInputInfo(ref LASTINPUTINFO plii);

        /// <summary>
        /// Return elapsed time since last user input.
        /// </summary>
        /// <returns></returns>
        static TimeSpan GetLastInputTime()
        {
            int idleTime = 0;
            LASTINPUTINFO lastInputInfo = new LASTINPUTINFO();
            lastInputInfo.cbSize = Marshal.SizeOf(lastInputInfo);
            lastInputInfo.dwTime = 0;

            int envTicks = Environment.TickCount;

            if (GetLastInputInfo(ref lastInputInfo))
            {
                int lastInputTick = (int)lastInputInfo.dwTime;

                idleTime = envTicks - lastInputTick;
            }

            return TimeSpan.FromSeconds((idleTime > 0) ? (idleTime / 1000) : 0);
        }

    }
}
