﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LockDownSvc 
{
    public partial class PasswordDlg : Form
    {
        public PasswordDlg()
        {
            InitializeComponent();
            Mode = PasswordMode.eValidate;
        }

        public string Password { set; get; }

        public enum PasswordMode { eValidate, eAccept };
        public PasswordMode Mode 
        { 
            get { return m_mode; }
            set
            {
                m_mode = value;
                switch (m_mode)
                {
                    case PasswordMode.eAccept:
                        this.Text = "Set new Password";
                        this.passwordTbox.Text = DecodePassword(Password);
                        this.passwordTbox.SelectAll();
                        break;
                    case PasswordMode.eValidate:
                        this.Text = "Enter Password to access Configuration";
                        break;
                }
            }
        }
        private PasswordMode m_mode;

        private void passwordTbox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                PasswordDone();
            }
        }

        private void cancelBtn_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.Close();
        }

        private void acceptBtn_Click(object sender, EventArgs e)
        {
            PasswordDone();
        }

        private bool PasswordDone()
        {
            switch (Mode)
            {
            case PasswordMode.eValidate:
                if (EncodePassword(passwordTbox.Text) == Password)
                {
                    this.DialogResult = System.Windows.Forms.DialogResult.OK;
                    this.Close();
                    return true;
                }
                else
                {
                    passwordTbox.Text = "Invalid";
                    passwordTbox.SelectAll();
                    return false;
                }
            case PasswordMode.eAccept:
                Password = EncodePassword(passwordTbox.Text);
                this.DialogResult = System.Windows.Forms.DialogResult.OK;
                this.Close();
                return true;
            }

            return false;
        }

        /// <summary>
        /// Encode(encrypt) clear text password.
        /// </summary>
        /// <param name="pwdInStr"></param>
        /// <returns></returns>
        internal string EncodePassword(string pwdInStr)
        {
            string pwdOutStr = string.Empty;
            byte mod = (byte)('z' - ' ') + 1;
            byte off = 11;
            if (pwdInStr != null)
            {
                foreach (char c in pwdInStr)
                {
                    byte cb = (byte)(c - 32);
                    if (cb >= mod)
                        throw new IndexOutOfRangeException();
                    char c2 = (char)(((cb + off) % mod) + 32);
                    pwdOutStr += c2;
                }
            }

            return pwdOutStr;
        }

        internal string DecodePassword(string pwdInStr)
        {
            string pwdOutStr = string.Empty;
            byte mod = (byte)('z' - ' ') + 1;
            byte off = (byte)(mod - 11);
            if (pwdInStr != null)
            {
                foreach (char c in pwdInStr)
                {
                    byte cb = (byte)(c - 32);
                    if (cb >= mod)
                        throw new IndexOutOfRangeException();
                    char c2 = (char)(((cb + off) % mod) + 32);
                    pwdOutStr += c2;
                }
            }

            return pwdOutStr;
        }

    }
}
