﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;
using System.Media;

namespace LockDownSvc
{                      
    /// <summary>
    /// Semi-transparent dialog displays time left and elapsed time.
    /// 
    /// Author: Dennis Lang
    /// LockDown service - Dec 2010
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>
    public partial class StatusDlg : Form
    {
        private ContextMenu m_ContextMenu = null;
        private System.Timers.Timer m_timer = new System.Timers.Timer(1000);
        private LockDownFg m_lockDown;
        private float m_opacity = 1.0f;

        public Config Configuration
        {
            get { return m_lockDown.Configuration; }
            set { m_lockDown.Configuration = value; }
        }

#if false
        private static Thread m_thread;

        /// <summary>
        /// Start the UI thread
        /// </summary>
        public static StatusDlg StartUIThread(LockDownFg lockDown)
        {
            StatusDlg dlg = new StatusDlg(lockDown);

            m_thread = new Thread(new ThreadStart(dlg.UIThread));
            m_thread.Start();

            return dlg;
        }

        /// <summary>
        /// UI thread
        /// </summary>
        public void UIThread()
        {
            Desktop desktop = new Desktop();
            if (!desktop.BeginInteraction())
            {
                return;
            }

            // Track mouse entering and leaving our UI.
            MouseMoveFilter mouseMoveFilter = new MouseMoveFilter();
            mouseMoveFilter.MouseMove += new EventHandler(MouseMoveEvent);
            Application.AddMessageFilter(mouseMoveFilter);

            Application.Run(this);
            Application.ExitThread();
        }


        /// <summary>
        /// Monitor mouse position, brighten UI when mouse cursor over UI, else default opacity.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MouseMoveEvent(object sender, EventArgs e)
        {
            MouseMoveFilter.MouseMoveEvent mme = (MouseMoveFilter.MouseMoveEvent)e;
            switch (mme.mouseAction)
            {
                case MouseMoveFilter.MouseMoveEvent.MouseAction.eEnter:
                    this.Opacity = 1.0f;
                    break;
                case MouseMoveFilter.MouseMoveEvent.MouseAction.eLeave:
                    this.Opacity = this.m_opacity;
                    break;
            }
        }

#endif

        public StatusDlg(LockDownFg lockDown)
        {
            m_lockDown = lockDown;
            InitializeComponent();
            this.Text = "LockDown v" + Application.ProductVersion;
            this.menuTitle.Text = this.Text;
            
            m_timer.Elapsed += new System.Timers.ElapsedEventHandler(Timer_ticked);
            m_timer.Enabled = true;
            Flash = false;
        }

        public void SetMsg(string msg, float opacity)
        {
            this.Text = msg;
            this.Opacity = m_opacity = opacity;
        }

        public bool Flash { get; set; }

        /// <summary>
        /// Move the window to the right-bottom corner
        /// </summary>
        /// <param name="e"></param>
        protected override void OnShown(EventArgs e)
        {
            this.Left = Screen.PrimaryScreen.WorkingArea.Left
                + Screen.PrimaryScreen.WorkingArea.Width
                - this.Width
                ;
            this.Top = Screen.PrimaryScreen.WorkingArea.Top
                + Screen.PrimaryScreen.WorkingArea.Height
                - this.Height
                ;
            UpdateUI();
        }

        protected override void OnClosed(EventArgs e)
        {
            m_timer.Enabled = false;
            this.notifyIcon.Visible = false;
            // Application.DoEvents();
        }
      
        #region ==== Timer, Update UI
        /// <summary>
        /// Update status every clock tick.
        /// </summary>
        void Timer_ticked(object sender, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                if (this.InvokeRequired)
                    this.Invoke(new UpdateUiDelegate(this.UpdateUI), null);
                else
                    this.UpdateUI();
            }
            catch { }
        }

        private delegate void UpdateUiDelegate();


        /// <summary>
        /// If configuration is for current user, update the status.
        /// If cursor inside UI, set opacity to full on.
        /// </summary>
        public void UpdateUI()
        {
            if (WinFunc.ForegroundUser == Configuration.User)
            {
                UpdateState();
            }
            else
            {
                leftTbox.Text = "?";
                leftUnits.Text = "";
            }

            if (this.Bounds.Contains(System.Windows.Forms.Cursor.Position))
            {
                this.Opacity = 1.0f;
                Flash = false;
            }
            else
            {
                if (Flash)
                    this.Opacity = (DateTime.Now.Second % 2);
                else
                    this.Opacity = m_opacity;
            }
        }

        private void UpdateState()
        {

            TimeSpan leftTs = Configuration.Left;
            if (Config.IsUnlimited(Configuration.Left))
            {
                leftTbox.Text = "Unlimited";
                leftUnits.Text = "";
            }
            else
            {
                if (leftTs.TotalHours > 1.0)
                {
                    leftTbox.Text = string.Format("{0}:{1:00}",
                        leftTs.Hours, leftTs.Minutes);
                    leftUnits.Text = "hh:mm";
                }
                else if (leftTs.TotalMinutes >= 0)
                {
                    leftTbox.Text = string.Format("{0}:{1:00}",
                        leftTs.Minutes, leftTs.Seconds);
                    leftUnits.Text = "mm:ss";
                }
                else
                {
                    leftTbox.Text = "Toast";
                    leftUnits.Text = "";
                }
            }

            TimeSpan elapsed = Configuration.Elapsed;
            if (elapsed.TotalHours > 1.0)
            {
                usedTbox.Text = string.Format("{0}:{1:00}",
                    elapsed.Hours, elapsed.Minutes);
                usedUnits.Text = "hh:mm";
            }
            else
            {
                usedTbox.Text = string.Format("{0}:{1:00}",
                    elapsed.Minutes, elapsed.Seconds);
                usedUnits.Text = "mm:ss";
            }
        }
        #endregion

        #region ==== UI creation, populate menus
       
        MenuItem m_openMenu;
        MenuItem m_closeMenu;
        MenuItem m_alarmMenu;

        private void SettingDlg_Load(object sender, EventArgs e)
        {
            m_ContextMenu = new ContextMenu();
            // m_ContextMenu.MenuItems.Add(new Menu
            m_openMenu = new MenuItem("Open LockDown", this.OpenDialog);
            m_ContextMenu.MenuItems.Add(m_openMenu);
            m_closeMenu = new MenuItem("Close LockDown", this.CloseDialog);
            m_ContextMenu.MenuItems.Add(m_closeMenu);

            m_ContextMenu.MenuItems.Add(new MenuItem("History", this.HistoryUI));
            m_alarmMenu = new MenuItem("Alarm", this.AlarmUI);
            m_ContextMenu.MenuItems.Add(m_alarmMenu);

            m_openMenu.Enabled = !(m_closeMenu.Enabled = true);
            configToolStripMenuItem.Visible = false;

            if (Config.IsAdmin(WinFunc.ForegroundUser))
            {
                configToolStripMenuItem.Visible = true;
                m_ContextMenu.MenuItems.Add(new MenuItem("Config", this.ConfigUI));
                m_ContextMenu.MenuItems.Add(new MenuItem("About", this.About));
                m_ContextMenu.MenuItems.Add(new MenuItem("LogOff", this.LogOff));
                m_ContextMenu.MenuItems.Add(new MenuItem("Exit", this.ExitGui));
            }

            // m_NotifyIcon = new NotifyIcon(new Container());
            this.notifyIcon.ContextMenu = m_ContextMenu;
            // this.notifyIcon.Icon = this.Icon;
            // this.notifyIcon.Visible = true;

            // m_NotifyIcon.BalloonTipText = "LockDown by Dennis Lang";
            // m_NotifyIcon.ShowBalloonTip(
            //   20, "LockDown", "(PC) Session Timer by Dennis Lang", ToolTipIcon.Info);
        }

        /// <summary>
        /// Open status dialog.
        /// </summary>
        public void OpenDialog(Object sender, EventArgs e)
        {
            this.Visible = true;
            BringToFront();
            m_openMenu.Enabled = !(m_closeMenu.Enabled = this.Visible);
        }

        /// <summary>
        /// Hide status dialog.
        /// </summary>
        public void CloseDialog(Object sender, EventArgs e)
        {
            this.Visible = false;
            m_openMenu.Enabled = !(m_closeMenu.Enabled = this.Visible);
        }

        protected override void WndProc(ref Message m)
        {
            const int WM_QUERYENDSESSION = 0x11;
            if (m.Msg == WM_QUERYENDSESSION)
            {
                // User has exited their session.
                m_lockDown.Action = LockDownFg.Actions.eEndSession;
            }

            base.WndProc(ref m);
        }

        /// <summary>
        /// Exit GUI (admin user only)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void ExitGui(object sender, EventArgs e)
        {
            this.Close();
            Application.Exit();
        }

        public void LogOff(object sender, EventArgs e)
        {
            LockDownFg.WriteBgCommand(LockDown.Fg2BgCommands.eLogOff);
        }

        public void About(object sender, EventArgs e)
        {
            AboutDlg aboutDlg = new AboutDlg();
            aboutDlg.Show();
        }

        private void HistoryUI(object sender, EventArgs e)
        {
            HistoryDlg historyDlg = new HistoryDlg(this.m_lockDown);
            historyDlg.ShowDialog(); // modal
        }

        AlarmDlg m_alarmDlg;
        private void AlarmUI(object sender, EventArgs e)
        {
            if (m_alarmDlg == null)
            {
                m_alarmDlg = new AlarmDlg();
                m_alarmDlg.AlarmTriggered += new EventHandler(AlarmTriggered);
                m_alarmDlg.AlarmStateChanged += new EventHandler(AlarmStateChanged);
                AlarmStateChanged(m_alarmDlg, EventArgs.Empty);
            }

            m_alarmDlg.Show();  // non-modal
        }

        private void AlarmTriggered(Object sender, EventArgs e)
        {
            this.Visible = true;
            this.BringToFront();
            Flash = true;
        }

        private void AlarmStateChanged(Object sender, EventArgs e)
        {
            this.notifyIcon.Icon = m_alarmDlg.AlarmEnabled ? m_alarmDlg.Icon : this.Icon;
            if (m_alarmDlg.AlarmEnabled)
                ShowTip("Alarm at " + m_alarmDlg.AlarmAt.ToString("H:mm:ss t"), false);
            else
                ShowTip(this.Text, false);
            Flash = false;
        }

        public void ConfigUI(Object sender, EventArgs e)
        {
            ConfigDlg configDlg = new ConfigDlg(LockDown.DataManager);

            // Catch any changes to self.
            configDlg.ConfigurationChanged += new EventHandler(ConfigChanged);
            configDlg.ShowDialog();         // modal - block until closed
        }

        private void ConfigChanged(object sender, EventArgs e)
        {
            ConfigDlg.ConfigEvent ce = e as ConfigDlg.ConfigEvent;
            if (ce != null)
            {
                Configuration = ce.m_cfg;
                UpdateState();
                LockDownFg.WriteBgCommand(LockDown.Fg2BgCommands.eLoadCfg);
            }
        }

        private void Ok_Click(object sender, EventArgs e)
        {
            this.Visible = false;
            m_openMenu.Enabled = !(m_closeMenu.Enabled = this.Visible);
        }

        private void usedTimeMenu_CheckedChanged(object sender, EventArgs e)
        {
            usedUnits.Visible = usedTbox.Visible = UsedLabel.Visible = usedTimeToolStripMenuItem.Checked;
            if (!usedTimeToolStripMenuItem.Checked)
                this.Height -= usedTbox.Height;
        }

        #endregion

        Rectangle m_notifyRegion = Rectangle.Empty;
        DateTime m_notifyDt = DateTime.MinValue;
        private void notifyIcon_MouseMove(object sender, MouseEventArgs e)
        {
            // this.notifyIcon.BalloonTipText = this.Text;
            // this.notifyIcon.ShowBalloonTip(1000);

            if (System.Windows.Forms.Control.MouseButtons == System.Windows.Forms.MouseButtons.None)
            {
                Point p = System.Windows.Forms.Control.MousePosition;
                if (m_notifyRegion == Rectangle.Empty)
                    m_notifyRegion = new Rectangle(p, new Size(1, 1));
                else
                    m_notifyRegion = Rectangle.Union(m_notifyRegion, new Rectangle(p, new Size(1, 1)));
                m_statusTip.Location = new Point(p.X - m_statusTip.Width / 4, p.Y - 50);
                timer.Enabled = true;
            }
            else
            {
                m_statusTip.Visible = false;
                m_notifyDt = DateTime.Now + TimeSpan.FromSeconds(3);
            }
        }

        StatusTipDlg m_statusTip = new StatusTipDlg();
        private void timerTip_Tick(object sender, EventArgs e)
        {
            if (DateTime.Now > m_notifyDt)
            {
                Point p = System.Windows.Forms.Control.MousePosition;
                timer.Enabled = m_statusTip.Visible = m_notifyRegion.Contains(p) &&
                    System.Windows.Forms.Control.MouseButtons == System.Windows.Forms.MouseButtons.None;
                m_statusTip.BringToFront();
            }
        }

        public void ShowTip(string msg, bool show)
        {
            m_statusTip.Message = msg;
            // timer.Enabled = show;
        }
    }
}
